/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.imda.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.tinygroup.imda.ModelInfomationGetter;
import org.tinygroup.imda.ModelLoader;
import org.tinygroup.imda.config.ModelDefine;
import org.tinygroup.imda.config.ModelProcessorDefine;
import org.tinygroup.imda.config.ModelProcessorStage;
import org.tinygroup.imda.exception.IMdaRuntimeException;
import org.tinygroup.springutil.SpringUtil;

public class ModelContainer {
	// key:modelInstance,value:modelDefine
	private Map<String, String> modelMap = new HashMap<String, String>();

	private Map<Class<?>, ModelDefine> modelDefineMap = new HashMap<Class<?>, ModelDefine>();

	private Map<String, Object> models = new HashMap<String, Object>();

	private Map<String, ModelDefine> modelDefines = new HashMap<String, ModelDefine>();

	private List<ModelLoader> modelLoaders = new ArrayList<ModelLoader>();
	
	public void addModelDefine(ModelDefine modelDefine) {
		modelDefines.put(modelDefine.getId(), modelDefine);
		String className = modelDefine.getModelClass();
		try {
			modelDefineMap.put(Class.forName(className), modelDefine);
		} catch (ClassNotFoundException e) {
			throw new IMdaRuntimeException("imda.modelClassNotFound",
					modelDefine.getId(), modelDefine.getModelClass());
		}
		String loaderBean = modelDefine.getModelLoaderBean();
		ModelLoader loader = SpringUtil.getBean(loaderBean);
		modelLoaders.add(loader);
	}

	public Map<String, Object> getModelInstances() {
		return models;
	}

	public Map<String, ModelDefine> getModelDefines() {
		return modelDefines;
	}

	/**
	 * 根据模型对象获取模型类型
	 * 
	 * @param model
	 *            模型对象
	 * @return 模型类型
	 */
	public ModelDefine getModelDefine(Object model) {
		Class<?> modelClass = model.getClass();
		for (Class<?> clazz : modelDefineMap.keySet()) {
			if (clazz.isAssignableFrom(modelClass)) {
				return modelDefineMap.get(clazz);
			}
		}
		throw new IMdaRuntimeException("imda.modelDefineNotFoundByClass",
				modelClass.getName());
	}

	/**
	 * 根据模型类型名获取模型类型
	 * 
	 * @param modelDefineName
	 *            模型类型名
	 * @return 模型类型
	 */
	public ModelDefine getModelDefine(String modelDefineName) {
		if (modelDefines.containsKey(modelDefineName))
			return modelDefines.get(modelDefineName);
		throw new IMdaRuntimeException("imda.modelDefineNotFound",
				modelDefineName);
	}

	/**
	 * 根据模型id获取模型类型
	 * 
	 * @param modelId
	 *            模型id
	 * @return 模型类型
	 */
	public ModelDefine getModelDefineByModelId(String modelId) {
		String modelDefineName = modelMap.get(modelId);
		if (modelDefines.containsKey(modelDefineName))
			return modelDefines.get(modelDefineName);
		throw new IMdaRuntimeException("imda.modelDefineNotFoundByModelId",
				modelId, modelDefineName);
	}

	/**
	 * 根据模型id、处理器名获取模型处理器
	 * 
	 * @param modelId
	 *            模型id
	 * @param processorName
	 *            模型类型的处理器名
	 * @return 模型处理器
	 */
	public ModelProcessorDefine getModelProcessorDefineByModelId(
			String modelId, String processorName) {
		ModelDefine define = getModelDefineByModelId(modelId);
		return getModelProcessorDefine(define, processorName);
	}

	/**
	 * 根据模型对象、处理器名获取模型处理器
	 * 
	 * @param model
	 *            模型对象
	 * @param processorName
	 *            模型类型的处理器名
	 * @return 模型处理器
	 */
	public ModelProcessorDefine getModelProcessorDefine(Object model,
			String processorName) {
		ModelDefine define = getModelDefine(model);
		return getModelProcessorDefine(define, processorName);
	}

	/**
	 * 获取模型类型的处理器
	 * 
	 * @param define
	 *            模型类型
	 * @param processorName
	 *            处理器名
	 * @return
	 */
	public ModelProcessorDefine getModelProcessorDefine(ModelDefine define,
			String processorName) {
		for (ModelProcessorDefine processorDefine : define
				.getModelProcessorDefines()) {
			if (processorDefine.getName().equals(processorName)) {
				return processorDefine;
			}
		}
		throw new IMdaRuntimeException("imda.processorDefineNotFound",
				define.getId(), processorName);
	}

	/**
	 * 根据模型、模型处理器名、处理器子阶段名获取处理器子阶段
	 * 
	 * @param modelId
	 *            模型
	 * @param processorName
	 *            模型处理器名
	 * @param stageName
	 *            模型处理器子阶段
	 * @return 模型处理器子阶段
	 */
	public ModelProcessorStage getModelProcessorStageByModelId(String modelId,
			String processorName, String stageName) {
		ModelProcessorDefine processorDefine = getModelProcessorDefineByModelId(
				modelId, processorName);
		return getModelProcessorStage(processorDefine, stageName);
	}

	/**
	 * 根据模型对象、模型处理器名、处理器子阶段名获取处理器子阶段
	 * 
	 * @param model
	 *            模型对象
	 * @param processorName
	 *            模型处理器名
	 * @param stageName
	 *            模型处理器子阶段名
	 * @return 模型处理器子阶段
	 */
	public ModelProcessorStage getModelProcessorStage(Object model,
			String processorName, String stageName) {
		ModelProcessorDefine processorDefine = getModelProcessorDefine(model,
				processorName);
		return getModelProcessorStage(processorDefine, stageName);
	}

	private ModelProcessorStage getModelProcessorStage(
			ModelProcessorDefine processorDefine, String stageName) {
		for (ModelProcessorStage stage : processorDefine.getProcessorStages()) {
			if (stage.getName().equals(stageName)) {
				return stage;
			}
		}
		throw new IMdaRuntimeException("imda.modelProcessorStageNotFound",
				processorDefine.getName(), stageName);
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	public void updateModel(Object model) {
		ModelInfomationGetter infoBean = getInfoBean(model);
		String id = infoBean.getId(model);
		models.put(id, model);

	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	public void addModel(Object model) {
		ModelInfomationGetter infoBean = getInfoBean(model);
		String id = infoBean.getId(model);
		models.put(id, model);
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	public void removeModel(Object model) {
		ModelInfomationGetter infoBean = getInfoBean(model);
		String id = infoBean.getId(model);
		models.remove(id);
	}

	@SuppressWarnings({ "rawtypes" })
	private ModelInfomationGetter getInfoBean(Object model) {
		ModelDefine modelDefine = getModelDefine(model);
		String infoBean = modelDefine.getModelInfomationGetterBean();
		return SpringUtil.getBean(infoBean);
	}

	public Object getModel(String modelId) {
		if (models.containsKey(modelId)) {
			return models.get(modelId);
		}
		throw new IMdaRuntimeException("imda.modelNotFound", modelId);
	}
	
	public List<ModelLoader> getModelLoaders() {
		return modelLoaders;
	}
}
